PRO FRAMECOPY, inmef, exten, result, TEXTLOG=textlog, DEBUG=debug

;+ ---------------------------------------------------------------------
; Program: FRAMECOPY v1.0    Author: Rolf A. Jansen (ASU) -- Apr 08 2010
;
; Procedure to extract a single image frame  from a Multi-Extension FITS
; file,  and write out a 2-D FITS format image just like 'autofilet.pro'
; does.
;
; ----------------------------------------------------------------------
; Input:   inmef   (string)   - Name of Multi-Extension FITS file
;          exten   (int)      - FITS extension to extract
; Output:  result  (string)   - Name of flat 2-D FITS format output file
;
; Optional input (and their defaults):
;          textlog (string)("framecopy.log") - Text log file
;          debug   (int)   (0)- Debugging flag for extra verbosity
;
; Example:
;          IDL> framecopy,"o6dc9b040_raw.fits",4,"o6dc9b040_rwc_4.fits"
;
; ----------------------------------------------------------------------
; USES readfits.pro,writefits.pro, sxpar.pro,sxaddpar.pro,sxdelpar.pro,
;      get_date.pro
; ----------------------------------------------------------------------
;
; Revision History:
;          v1.0 -- Apr  8 2010 (RAJ) Extracted relevant routines from
;                  'autofilet.pro' by R.A. Jansen.
;
;- ----------------------------------------------------------------------

progver='FRAMECOPY v1.0 - R.A. Jansen - (IDL 6.3)'

; CHECK THE NUMBER OF COMMAND LINE ARGUMENTS...
IF (n_params(0) LT 3) THEN BEGIN
  print,'SYNTAX:  AUTOFILET, "inmef", exten=#, result="result"'
  print,'  where "inmef" is the name of a Multi-Extension FITS file, # is the'
  print,'  number of the FITS extension to extract,  and "result" is the name'
  print,'  of the flat 2-D FITS format output file to create.'
  print,'OPTIONAL ARGUMENTS:'
  print,'  [textlog="aaaaaaaa"] [debug=(0|1)]'
  RETURN
ENDIF

; PROGRAM DEFAULTS...
IF (NOT keyword_set(textlog)) THEN textlog ="framecopy.log"
IF (NOT keyword_set(debug))   THEN debug   = 0     ; debugging level (0|1)

; CHECK INPUT PARAMETERS (assume negative values to be typos)...
IF (exten LT 0)  THEN exten = abs(exten)


; ----------------------------------------------------------------------

T0  = SYSTIME(1)

; PADD FILE NAME EXTENSION, IF MISSING...
imtyp = ".fits"
cext  = STRMID(inmef,(STRPOS(inmef,'.',/REVERSE_SEARCH)+1))
; Valid image format extensions are <=4 characters in length; if more,
; assume it is part of the rootname and that the extension is missing:
IF (STRLEN(cext) LE 4 AND cext NE "fits") THEN BEGIN
   print,'ERROR: image extension ".'+cext+'" is not ".fits"! Exit...'
   RETURN
ENDIF
IF (STRLEN(cext) GT 4) THEN inmef = inmef+imtyp

; For output images, append ".fits" if not already the extension:
cext = STRMID(result,STRPOS(result,'.',/REVERSE_SEARCH)+1)
IF (cext NE "fits") THEN result = result+imtyp


; READ THE FITS PRIMARY HEADER UNIT, AND CHECK WHETHER "EXTEND == T" ...
phu  = READFITS(inmef,primehead,exten=0,/silent)
xtnd = STRTRIM(SXPAR(primehead,'EXTEND'))
; READ MJD AT THE START OF THE 1ST EXPOSURE FROM THE PRIMARY HEADER UNIT...
tmjd = STRTRIM(SXPAR(primehead,'TEXPSTRT'))

; READ THE (SPECIFIED EXTENSION IN THE) FITS FILE...
IF xtnd THEN BEGIN
   img  = READFITS(inmef,extenhead,exten=exten,/silent)
   xtn  = '['+STRTRIM(SXPAR(extenhead,'EXTNAME'))+']'
   cxtn = '['+STRTRIM(exten,1)+'="'+STRTRIM(SXPAR(extenhead,'EXTNAME'))+'"]'
ENDIF ELSE BEGIN
   img  = READFITS(inmef,primehead,/silent)
   xtn  = ''
ENDELSE

; DETERMINE THE IMAGE SIZE (number of columns and lines)...
image_sz = SIZE(img)
ncol = image_sz[1]
nrow = image_sz[2]

; OPEN TEXT LOGFILE...
OPENW,25,textlog,/APPEND

; PRINT OPENING MESSAGE...
printf,25,'FRAMECOPY ',inmef,cxtn,' --> ',result,f='(a,a,a,a,a)'
print,'FRAMECOPY ',inmef,cxtn,' --> ',result,f='(a,a,a,a,a)'

; ----------------------------------------------------------------------

; GENERATE OUTPUT IMAGE (convert to floating point)...
outimg = FLTARR(ncol,nrow)
FOR row=0,nrow-1 DO BEGIN
   outimg[*,row] = img[*,row]
ENDFOR

; ----------------------------------------------------------------------

; WRITE OUTPUT FITS IMAGE(S)...
GET_DATE,dte,/TIMETAG

; First update primary header to conform to the FITS standard...
SXADDPAR,primehead,'SIMPLE','T',' FITS standard'
SXADDPAR,primehead,'BITPIX',-32,' Bits per data value'
SXADDPAR,primehead,'NAXIS',2,' Number of data axes'
SXADDPAR,primehead,'NAXIS1',ncol,' Length of axis 1',BEFORE='EXTEND'
SXADDPAR,primehead,'NAXIS2',nrow,' Length of axis 2',BEFORE='EXTEND'
SXADDPAR,primehead,'EXTEND','F',' File may contain standard extensions'
SXDELPAR,primehead,'NEXTEND'
SXDELPAR,primehead,'GROUPS'
SXADDPAR,primehead,'ORIGIN',progver,' FITS file originator',BEFORE='DATE'
SXADDPAR,primehead,'DATE',dte,' Date this file was written'
SXADDPAR,primehead,'FILENAME',STRMID(result,STRPOS(result,'/',/REVERSE_SEARCH)+1),' Name of FITS file'
; ideally, one would change 'OMIT' to 'FAILED' but this is not a valid value...
SXADDPAR,primehead,'HPATCORR','OMIT',' Remove herring-bone pattern noise',BEFORE='DQICORR'
SXDELPAR,primehead,'END'
SXDELPAR,primehead,'HISTORY'
SXDELPAR,primehead,'COMMENT'

; Delete extension header specific keywords...
SXDELPAR,extenhead,'XTENSION'
SXDELPAR,extenhead,'BITPIX'
SXDELPAR,extenhead,'NAXIS'
SXDELPAR,extenhead,'NAXIS1'
SXDELPAR,extenhead,'NAXIS2'
SXDELPAR,extenhead,'PCOUNT'
SXDELPAR,extenhead,'GCOUNT'
SXDELPAR,extenhead,'INHERIT'
SXDELPAR,extenhead,'EXTNAME'
SXDELPAR,extenhead,'EXTVER'
SXDELPAR,extenhead,'DATE'
SXDELPAR,extenhead,'ORIGIN'

; Merge remaining keywords in extension header with primary header...
newhead = [primehead,extenhead]
SXADDPAR,newhead,'BZERO',0.0,' physical value for an array value of zero'
;; the following keyword is also present in all frames processed with
;; 'autofilet.pro', so we have to keep it here and deal with it elsewhere..
;SXDELPAR,newhead,'O_BZERO'

; Write the flat, 2-D FITS format output file...
WRITEFITS,result,outimg,newhead

CLOSE,25,/ALL,/FORCE

RETURN
END

;-------------------------------  R A J  -------------------------------
