C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfgkwrd (crec, ckey, ilen, lfix, lcom, idbg, ierr)

C @(#)Subroutine: rjfgkwrd           Author: R.A. Jansen -- Sep 14, 2009
C @(#)
C @(#)  Purpose: Read a header keyword 'ckey' from a 80-char FITS header
C @(#)    record 'crec', verify that it complies with the FITS standard,
C @(#)    and  return its actual  (rather than declared)  length through
C @(#)    'ilen'.  If boolean switch  'lfix'  is true, attempt to repair
C @(#)    non-compliant keywords.  Any keyword that is not followed by a
C @(#)    value indicator ("= " in bytes 9:10)  will be interpreted as a
C @(#)    comment,  causing flag 'lcom' to be returned true on exit.  If
C @(#)    verbosity level 'idbg'>0,  additional messages may be printed.
C @(#)    If any error is detected, error status 'ierr' will be non-zero
C @(#)    upon exit.  Its value  (if >2000)  uniquely identifies a known
C @(#)    known error condition. The FITS standard requires a keyword to
C @(#)    be a  left-justified,  8-character,  space-padded ASCII string
C @(#)    without embedded spaces,  composed of uppercase letters (A-Z),
C @(#)    digits (0-9), hyphen (-) and underscore (_). The first charac-
C @(#)    ter must be  an uppercase letter  (or a space,  in the special
C @(#)    case of blank keywords that are not followed  by a value indi-
C @(#)    cator).  For indexed keyword names,  the counter must not have
C @(#)    leading zeros.  If 'lfix' was true on invokation and a keyword
C @(#)    was indeed fixed,  then 'crec' will be modified too upon exit.
C @(#)    If the record is a comment record (ie., no "= " in bytes 9:10)
C @(#)    but byte 9 is not a blank,  then the comment in 'crec' will be
C @(#)    shifted to the right by one byte upon exit.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL rjfgkwrd(crec, ckey, ilen, lfix, lcom, idbg, ierr), where
C @(#)    'crec' and 'ckey' are strings with declared lengths of 80  and
C @(#)    8 chars, respectively,  and 'ilen' is an integer.  'lfix' is a
C @(#)    boolean switch, and 'lcom' is boolean flag.  'idbg' and 'ierr'
C @(#)    are both integers.
C @(#)
C @(#)  Dependencies:     cinit, clen, ucase, parse
C @(#)
C @(#)  Revision history:
C @(#)    Jun 11, 2009 [RAJ] -- Split off repeat code from 'chkfits' and
C @(#)                          'exthdr';  generalized it as a low-level
C @(#)                          routine;  implemented fixing of keywords
C @(#)                          that violate the strict FITS standard.
C @(#)    Jun 12, 2009 [RAJ] -- Changed test for value indicator ("= ");
C @(#)                          added 'lcom' to calling sequence to flag
C @(#)                          comment records (blank in character 9).
C @(#)    Jun 19, 2009 [RAJ] -- Found FITS files with comments  starting
C @(#)                          in char 9 i.s.o. 10!   Modified the code
C @(#)                          accordingly & updated the documentation.
C @(#)    Sep 14, 2009 [RAJ] -- Modified to adhere to naming conventions
C @(#)                          of more recent header routines;  changed
C @(#)                          error codes.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER	crec*80, ckey*8
      INTEGER	ilen, idbg, ierr
      LOGICAL	lfix, lcom

C --- Declaration of local variables...
      CHARACTER ctmp*80, dbid*19, cc, cf
      INTEGER	clen, j, ic

C     'iA','iZ','i0','i9','ih','iu','ie' and 'ib' are shorthand integer
C     equivalents in the ASCII collating sequence of uppercase A and Z,
C     digits 0 and 9, hyphen, underscore, =, and space, respectively...
      INTEGER	iA, iZ, i0, i9, ih, iu, ie, ib

C     Flag 'blank' is raised when the first character is a space..
      LOGICAL	blank

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the debugging ID string...
      dbid  = 'DEBUG(rjfgkwrd): '
 
C --- Initialize the output variables...
      CALL cinit(ckey)
      ilen = -1
      lcom = .FALSE.

C --- Initialize the local variables...
      CALL cinit(ctmp)
      iA = ICHAR('A')
      iZ = ICHAR('Z')
      i0 = ICHAR('0')
      i9 = ICHAR('9')
      ih = ICHAR('-')
      iu = ICHAR('_')
      ie = ICHAR('=')
      ib = ICHAR(' ')
      blank = .FALSE.


C --- Test whether the 1st character in the record starts with either an
C     uppercase letter or a blank (comment lines for formatting)...
      cc = crec(1:1)
      ic = ICHAR(cc)

      IF (ic.GE.iA.AND.ic.LE.iZ) THEN
C        First character of record (hence keyword) is a capital letter
         CONTINUE
      ELSE IF (ic.EQ.ib) THEN
C        First character is a blank (this is likely a comment line)
         blank = .TRUE.
      ELSE
C        First character of record (and keyword) is not a capital letter
         IF (lfix) THEN
            CALL ucase(cc,cf)
            IF (cf.EQ.cc) THEN
C              First character is not a letter and is not fixable...
               IF (idbg.GT.0) WRITE(*,*)
     +            dbid//'keyword does not start with a letter'
               ierr = 2601
               GOTO 9
            ELSE
C              First character was a lowercase letter and was fixable...
               ctmp = crec
               crec(1:1) = cf
               IF (idbg.GT.1) WRITE(*,*) dbid//' (lfix = T)  "'//
     +            ctmp(1:8)//'"-->"'//crec(1:8)//'"'
            ENDIF
         ELSE
            IF (idbg.GT.0) WRITE(*,*)
     +         dbid//'keyword does not start with an uppercase letter'
            ierr = 2602
            GOTO 9
         ENDIF
      ENDIF


C --- Try to parse the current record in a keyword and value(+comment);
C     if the first character was a blank, the 9th character must not be
C     an "="-sign.  Test that characters 2--8  comply with the standard
C     and that character 9 is either an "="-sign (in which case char 10
C     must be a blank), or a blank or uppercase character (A-Z) ...
      ctmp = crec
      CALL parse(ctmp,'=',ckey,ilen)

      IF (blank.AND.ilen.EQ.0) THEN
C        Test that character 9 is not an '=' sign...
         IF (crec(9:9).EQ.'=') THEN
            IF (idbg.GT.0) WRITE(*,*) dbid//
     +         'invalid blank keyword with an "=" sign in character 9'
            ierr = 2603
            GOTO 9
         ELSE
C           OK, blank keyword and no value indicator: comment record
            IF (idbg.GT.1) WRITE(*,*) dbid//' blank keyword  [ok]'
            lcom = .TRUE.
         ENDIF

      ELSE IF (blank) THEN
C        Keyword starts with a blank followed by non-blanks: invalid!
         IF (idbg.GT.0) WRITE(*,*)
     +      dbid//'invalid keyword starting with a blank character'
         ierr = 2604
         GOTO 9

      ELSE
C        Test each of characters 2--8 for compliance with the standard..
         ctmp = crec
         DO j=2,ilen,1
            cc = crec(j:j)
            ic = ICHAR(cc)
            IF ( (ic.GE.iA.AND.ic.LE.iZ) .OR. (ic.GE.i0.AND.ic.LE.i9)
     +          .OR. ic.EQ.ih .OR. ic.EQ.iu ) THEN
C             character is compliant with the FITS standard...
              CONTINUE
            ELSE IF (ic.EQ.ib) THEN
C             keyword has invalid embedded blank character(s)...
              IF (idbg.GT.0) WRITE(*,*) dbid//'invalid keyword '//
     +           'with embedded blank(s)'
              ierr = 2605
              GOTO 9
            ELSE
C             keyword may contain lowercase letter or invalid character.
              IF (lfix) THEN
                 CALL ucase(cc,cf)
                 IF (cf.EQ.cc) THEN
C                   character was not a letter and is not fixable...
                    IF (idbg.GT.0) WRITE(*,*) dbid//'invalid keyword'//
     +                 ' with embedded non-printable character(s)'
                    ierr = 2606
                    GOTO 9
                 ELSE
C                   character was lowercase letter and could be fixed...
                    crec(j:j) = cf
                    IF (idbg.GT.1) WRITE(*,*) dbid//' (lfix = T)  "'//
     +                 ctmp(1:8)//'"-->"'//crec(1:8)//'"'
                 ENDIF
              ELSE
                 IF (idbg.GT.0) WRITE(*,*) dbid//'invalid keyword wi'//
     +            'th embedded lower case or non-printable character(s)'
                 ierr = 2607
                 GOTO 9
              ENDIF
            ENDIF

         ENDDO

      ENDIF


C --- Test that if we're dealing with an indexed keyword, there are no
C     leading zeros; if there are and 'lfix', then try to fix...
      IF (ilen.GT.0) THEN
         cc = crec(ilen:ilen)
         ic = ICHAR(cc)
         IF (ilen.GT.2.AND.ic.GE.i0.AND.ic.LE.i9) THEN
C           This looks like it might be an indexed keyword; find last
C           non-digit (we know the first character is not a digit)...
            DO j=ilen-1,1,-1
               cc = crec(j:j)
               ic = ICHAR(cc)
               IF (.NOT.(ic.GE.i0.AND.ic.LE.i9)) GOTO 1
            ENDDO
1           CONTINUE
2           IF ((ilen-1-j).GT.0) THEN
C              There were additional digits; test for a leading zero...
               cc = crec((j+1):(j+1))
               ic = ICHAR(cc)
               IF (ic.EQ.i0.AND.lfix) THEN
C                 Indexed keyword has a leading zero: try and fix...
                  ctmp(1:8) = crec(1:8)
                  crec((j+1):(ilen-1)) = crec((j+2):ilen)
                  crec(ilen:ilen) = ' '
                  IF (idbg.GT.1) WRITE(*,*) dbid//' (lfix = T)  "'//
     +               ctmp(1:8)//'" --> "'//crec(1:8)//'"'
                  IF (idbg.GT.1) WRITE(*,*)dbid//' ilen =',ilen,', j=',j
                  ilen = ilen - 1
                  GOTO 2
               ELSE IF (ic.EQ.i0) THEN
C                 Indexed keyword has a leading zero: not OK.
                  IF (idbg.GT.1) WRITE(*,*) dbid//' invalid keyword: '//
     +               'leading zero(s) in indexed keyword.'
                  ierr = 2608
                  GOTO 9
               ELSE
C                 Index number starts with a non-zero digit: OK.
                  ierr = 0
               ENDIF
            ENDIF
         ENDIF
      ENDIF


C --- Test characters 9 and 10 for value indicator ("= "); raise flag if
C     this is a comment card...
      cc = crec(9:9)
      ic = ICHAR(cc)
      IF (ic.EQ.ie) THEN
C        "="-sign in position 9: regular keyword+value pair in card...
         cc = crec(10:10)
         ic = ICHAR(cc)
         IF (ic.NE.ib) THEN
C           invalid character in position 10 following "="-sign...
            IF (idbg.GT.0) WRITE(*,*) dbid//'invalid character '//
     +         'in position 10 following the "=" sign'
            ierr = 2609
            GOTO 9
         ENDIF

      ELSE IF ((ic.GE.iA.AND.ic.LE.iZ) .OR. ic.eq.ib ) THEN
C        blank or uppercase character in position 9: interpret this as a
C        comment record...
         lcom = .TRUE.
         IF (ic.NE.ib) THEN
C           mildly non-compliant with original FITS standard (allowed in
C           FITS v3.0): comment starts 1 character too early...
            ctmp = crec
            IF (lfix) THEN
               crec(9:9) = ' '
               crec(10:80) = ctmp(9:79)
               IF (idbg.GT.1) WRITE(*,*) dbid//' (lfix = T)  "'//
     +                 ctmp(1:12)//'.."-->"'//crec(1:12)//'.."'
            ELSE
C              invalid char in position 9 (neither an "=" nor a blank)..
               IF (idbg.GT.0) WRITE(*,*) dbid//'invalid character '//
     +            'in position 9: neither "=" sign nor a blank'
               ierr = 2610
               GOTO 9
            ENDIF
         ENDIF

      ELSE
C        invalid character in position 9 (neither an "=" nor a blank)...
         IF (idbg.GT.0) WRITE(*,*) dbid//'invalid character '//
     +      'in position 9: neither "=" sign nor a blank'
         ierr = 2611
         GOTO 9
      ENDIF      


C --- Remove the value indicator (if any) in case the keyword matches
C     one of the reserved comment keywords ("HISTORY" and "COMMENT"),
C     then return the keyword, the actual length thereof, and the
C     possibly modified input header record...
      ckey = crec(1:8)
      IF (ckey.EQ.'HISTORY '.OR.ckey.EQ.'COMMENT ') crec(9:9) = ' '


9     IF (idbg.GT.2) WRITE(*,*) dbid//crec(1:clen(crec))
      IF (idbg.GT.1) WRITE(*,*) dbid//' (lfix =',lfix,').  On exit, '//
     +   'ckey="'//ckey//'"'
      IF (idbg.GT.1) WRITE(*,*) '                     ilen=',ilen,
     +   ', lcom=',lcom,', ierr=',ierr

      RETURN
      END


